/*
  EZinst - Easy Installation -
  Copyright (C) 1993, 1994, 1995 Takashi MANABE
                                     (manabe@Roy.dsl.tutics.tut.ac.jp)

  EZinst is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include "paths.h"
#include "label.h"
#include "screen.h"
#include "menu.h"
#include "buffer.h"
#include "message.h"
#include "list.h"
#include <jcurses.h>

#define	EXT_NAME	".tgz"

static DIR     *currentDir;

extern int categoryLabels, packageLabels;
extern struct labelLink *labelHead;
extern char *packageDirs[];
static struct labelLink *packageHead;
static int installPackages, totalSize;

int DirNext(char *name)
{
    struct dirent *dent;

    if (currentDir == NULL) return(FALSE);

    if ((dent = readdir(currentDir)) != NULL) {
	strcpy(name, dent->d_name);
	return(0);
    }
    closedir(currentDir);
    currentDir = NULL;
    return(EOF);
}

int DirFirst(char *dir, char *name)
{
    currentDir = opendir(dir);
    return(DirNext(name));
}

static char *PackagesList(char *mesg)
{
    int num, size;
    char *lspkg, *p;
    struct labelLink *llp;

    num = installPackages * 10 + (mesg ? strlen(mesg): 0);
    p = lspkg = malloc(num);
    if (mesg) {
	strcpy(lspkg, mesg);
	p += strlen(mesg);
    }
    num = 0;
    for (llp = packageHead; llp != NULL; llp = llp->next)
	if (llp->value == TRUE && llp->selected != TRUE) {
	    sprintf(p, "%-9s", llp->name);
	    num ++;
	    p += 9;
	    if (num * 9 >= COLS - 10) {
		if ((num % 2) != 0) p --;
		*p = '\n';
		p ++;
		num = 0;
	    }
	    size += llp->size;
	}
    if ((num % 2) != 0) p --;
    if (*(p-1) != '\n') {
	*p = '\n';
	p ++;
    }
    *p = '\0';
    return(lspkg);
}

static int LsPackages(struct menuLink *pmp)
{
    char *lspkg;

    lspkg = PackagesList(NULL);
    MessageWin(lspkg, pmp->desc, 0);
    free(lspkg);
    return(0);
}

static int RunCommand(WINDOW *win, char *cmd)
{
    FILE *fp;
    char line[256];

/*
    wprintw(win, ">%s\n", cmd);
    wrefresh(win);
*/
    sprintf(line, "%s 2>&1", cmd);
    if ((fp = popen(line, "r")) == NULL) return(-1);
    while (fgets(line, 255, fp)) {
	wprintw(win, line);
	wrefresh(win);
    }
    return(pclose(fp));
}

static inline
struct labelLink *IsInstall(char *name)
{
    struct labelLink *llp;

    for (llp = packageHead; llp != NULL; llp = llp->next) {
	if (llp->value == TRUE && !llp->selected
	    && !strcmp(llp->name, name)) {
	    return(llp);
	}
    }
    return(NULL);
}

static
int InstallPackage(WINDOW *win, char *dir, char *name)
{
    int r;
    struct labelLink *llp;
    char line[256], *p;

    if ((p = strchr(name, '.')) != NULL) *p = '\0';
    llp = IsInstall(name);
    if (p) *p = '.';
    if (llp == NULL) return(0);
    sprintf(line, PATH_EZPKG" %s/%s", dir, name);
    if ((r = RunCommand(win, line)) == 0) {
/*	installPackages --;*/
	llp->selected = TRUE;
    }
    installPackages --; /* whether succeed or not */
    return(llp->selected ? llp->size: 0);
}

static
int InstallDir(WINDOW *win, WINDOW *mwin, char *dir, int size, int maxx)
{
    int x, y, r;
    char name[20];

    if (DirFirst(dir, name)) return(size);
    do {
	if (!strstr(name, EXT_NAME)) continue;
	r = InstallPackage(mwin, dir, name);
	if (r > 0) {
	    size += r / 1024;
	    y = (size * 100) / totalSize;
	    wmove(win, 0, 0);
	    wstandout(win);
	    for (x = 0; x < (y * maxx)/100 ; x ++) waddch(win, ' '); 
	    wstandend(win);
	    mvwprintw(win, 0, maxx, "<%3d%%", y);
	    wrefresh(win);
	}
    } while (!DirNext(name));
    return(size);
}

static
int DoInstall(struct menuLink *pmp)
{
    int size, r, maxx, i;
    char *lspkg, line[256];
    WINDOW *win, *mwin;
    struct labelLink *llp;
    extern char *devFd;
    extern short fromFloppy;

    if (fromFloppy) MessageWin(GetMessage(MSG_INSTALL_FLOPPY),
			       "Install from  floppy disks", 0);
    clear();
    refresh();
    size = 0;
    mwin = newwin(LINES - 3, COLS, 3, 0);
    scrollok(mwin, TRUE);
    maxx = ((COLS - 4) / 2) * 2;
    win = OpenWin(1, maxx, 0, 0, 0, 0, 0, 0, "", "");
    maxx -= 10;
    if (fromFloppy) { /* from floppy disk */
	while (installPackages) {
	    wclear(win);
	    mvwprintw(win, 0, 0, "%s", GetMessage(MSG_PRESSRET));
	    BEEP(1);
	    wrefresh(win);
	    r = 0;
	    do {
		switch(GetKey(win)) {
		case 'Q':
		case 'q':
		    r = -1;
		    break;
		case '\r':
		    r = 1;
		    break;
		}
	    } while (!r);
	    if (r < 0) break;
	    sprintf(line, _PATH_MOUNT" -rt msdos %s "DIR_MOUNT, devFd);
	    wclear(win);
	    mvwprintw(win, 0, 0, "%s", GetMessage(MSG_NOW_MOUNTING));
	    wrefresh(win);
	    if ((r = RunCommand(mwin, line)) == 0) {
		size = InstallDir(win, mwin, DIR_MOUNT, size, maxx);
		r = RunCommand(mwin, _PATH_UMOUNT" "DIR_MOUNT);
	    }
	}
    } else for (i = 0; packageDirs[i] && installPackages > 0; i ++) {
#if 1
	size = InstallDir(win, mwin, packageDirs[i], size, maxx);
#else
	if (DirFirst(packageDirs[i], name)) continue;
	do {
	    if (!strstr(name, EXT_NAME)) continue;
	    r = InstallPackage(mwin, packageDirs[i], name);
	    if (r > 0) {
		size += r / 1024;
		y = (size * 100) / totalSize;
		wmove(win, 0, 0);
		wstandout(win);
		for (x = 0; x < (y * maxx)/100 ; x ++) waddch(win, ' '); 
		wstandend(win);
		mvwprintw(win, 0, maxx, "<%3d%%", y);
		wrefresh(win);
	    }
	} while (!DirNext(name));
#endif
    }
    wclear(win);
    mvwprintw(win, 0, 0, GetMessage(MSG_INST_SETUP));
    wrefresh(win);
    sprintf(line, PATH_EZPKG" -setup");
    RunCommand(mwin, line);
    wclear(win);
    mvwprintw(win, 0, 0, GetMessage(MSG_PRESSKEY));
    BEEP(3);
    wrefresh(win);
    getchar();
    CloseWin(win);
    delwin(mwin);
    RefreshAll();
    for (llp = packageHead; llp != NULL; llp = llp->next)
	if (llp->value == TRUE && llp->selected != TRUE) break;

    if (llp != NULL) {
	lspkg = PackagesList(GetMessage(MSG_NOTINST_YET));
	r = MessageWin(lspkg, "", 0);
	free(lspkg);
	if (r == 'y') {
	    ListSave(packageHead, SAVE_NOTINST);
	}
    }
    return(0);
}

#ifdef	INSTALL_SOURCE
static
int LsSource(struct menuLink *pmp)
{
    struct menuLink *mlp, *tail, *head;
    return(0);
}
#endif	INSTALL_SOURCE

int InstallMenu(struct menuLink *pmp)
{
    int num;
    struct menuLink *mlp, *tail, *head;
    struct labelLink *llp;

    num = 0;
    for (llp = labelHead; llp != NULL; llp = llp->next) {
	if (num >= categoryLabels) break;
	num ++;
    }
    packageHead = llp;
    totalSize = num = 0;
    for (llp = packageHead; llp != NULL; llp = llp->next)
	if (llp->value == TRUE) {
	    num ++;
	    llp->selected = FALSE;
	    totalSize += llp->size;
	}
    totalSize /= 1024;
    if (num <= 0) return(0);
    installPackages = num;
    head = tail = NULL;
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_INST_CONFIRM);
    mlp->key = 'c';
    mlp->able = TRUE;
    mlp->p.run = LsPackages;
#ifdef	INSTALL_SOURCE
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_INST_CHECKLS);
    mlp->key = 'l';
    mlp->able = TRUE;
    mlp->p.run = LsSource;
#endif	INSTALL_SOURCE
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_INST_START);
    mlp->key = 's';
    mlp->able = TRUE;
    mlp->p.run = DoInstall;
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_QUIT);
    mlp->key = 'q';
    mlp->able = TRUE;
    MenuWin(head, pmp->desc, 1, 2);
    DEL_LINK(struct menuLink, head);
    return(0);
}
