/*
  EZinst - Easy Installation -
  Copyright (C) 1993, 1994, 1995 Takashi MANABE
                                     (manabe@Roy.dsl.tutics.tut.ac.jp)

  EZinst is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include "paths.h"
#include "label.h"
#include "screen.h"
#include "menu.h"
#include "buffer.h"
#include "message.h"
#include "list.h"
#include <jcurses.h>

static int GetLine(WINDOW *win, char *line)
{
    char ch, *p;

    p = line + strlen(line);
    while (1) {
	wclear(win);
	mvwprintw(win, 0, 0, line);
	wrefresh(win);
	if (strchr("\r\n\x4", ch = wgetch(win))) break;
	if (ch == 0x08) {
	    if (p > line) {
		p --;
		*p = '\0';
	    }
	} else {
	    *p = ch;
	    if ((p - line) < win->maxx) p ++;
	}
    }
    return(p - line);
}

int ListSave(struct labelLink *head, int mode)
{
    struct labelLink *llp;
    int x, size;
    time_t now;
    FILE *fp;
    char line[256];
    WINDOW *win;

    x = ((COLS - 4) / 2) * 2;
    if (x > 46) x = 46;
    win = OpenWin(1, x, 0, 0, 0, 0, 0, 0,
		  GetMessage(MSG_INPUT_COMMENT), "");
    time(&now);
    sprintf(line, PATH_LIST"/%08x.%s", now,
	    (mode == SAVE_INSTALL) ? "lst": "yet");
    if ((fp = fopen(line, "w")) == NULL) {
	mvwprintw(win, 0, 0, "%s", GetMessage(MSG_MFILE_ERR));
	wrefresh(win);
	getchar();
	CloseWin(win);
	return(-1);
    }
    do {
	wmove(win, 0, 0);
	wrefresh(win);
	memset(line, 0, 256);
	x = GetLine(win, line);
	fprintf(fp, "# %s\n", line);
    } while (x != 0);
    for (size = 0, llp = head; llp != NULL; llp = llp->next)
	if (llp->value == TRUE
	    && (mode == SAVE_INSTALL || llp->selected != TRUE)) {
	    size += llp->size;
	}
    fprintf(fp, "$ %d\n", size);
    for (llp = head; llp != NULL; llp = llp->next)
	if (llp->value == TRUE
	    && (mode == SAVE_INSTALL || llp->selected != TRUE)) {
	    fprintf(fp, "%s\n", llp->name);
	}
    fclose(fp);
    CloseWin(win);
    return(0);
}

int ListLoad(char *name)
{
    FILE *fp;
    char *p, line[256];
    struct labelLink *llp;

    if ((fp = fopen(name, "r")) == NULL) return(-1);
    while (fgets(line, 255, fp)) {
	if (line[0] == '#' || line[0] == '$') continue;
	if ((p = strpbrk(line, ".\r\n")) != NULL) *p = '\0';
	if ((llp = GetLabelFromName(line)) != NULL) {
	    llp->value = TRUE;
	    llp->selected = FALSE;
	}
    }
    fclose(fp);
}

int ListCatalog(struct menuLink *pmp)
{
    int i=0;
    FILE *fp;
    struct menuLink *mlp, *tail, *head;
    struct labelLink *llp, *ltail, *lhead;
    char *name, *path, buff[1024], *p, line[51];

    lhead = ltail = NULL;
    head = tail = NULL;
    path = malloc(strlen(PATH_LIST) + 20);
    strcpy(path, PATH_LIST"/");
    name = path + strlen(path);
    if (DirFirst(PATH_LIST, name)) return(0);
    do {
	if (!strstr(name, ".lst") && !strstr(name, ".yet")) continue;
	if ((fp = fopen(path, "r")) == NULL) continue;
	i ++;
	NEW_LINK(struct menuLink, head, tail, mlp);
	NEW_LINK(struct labelLink, lhead, ltail, llp);
	buff[0] = '\0';
	while (fgets(line, 50, fp) && line[0] == '#') {
	    p = line;
	    while (strchr("# \t", *p)) p ++;
	    if (!strchr("\r\n", *p)) strcat(buff, p);
	}
	while (line[0] != '$' && fgets(line, 50, fp));
	if (line[0] == '$') {
	    p = line;
	    while (strchr("$ \t", *p)) p ++;
	    llp->size = atoi(p);
	}
	sprintf(line, "%5d [KB]\n", llp->size / 1024);
	strcat(buff, line);
	llp->desc = strdup(buff);
	llp->value = FALSE;
	mlp->p.lbl = llp;
	mlp->desc = strdup(name);
	mlp->key = '0' + i;
	mlp->able = TRUE;
	mlp->key = ' ';
	fclose(fp);
    } while (!DirNext(name));

    if (i > 0) {
	SelectWin("", head, "", 66, SELECT_LIST);
	for (mlp = head; mlp != NULL; mlp = mlp->next) {
	    if (mlp->p.lbl->value) {
		strcpy(name, mlp->desc);
		ListLoad(path);
	    }
	    free(mlp->desc);
	    free(mlp->p.lbl->desc);
	}
    }
    DEL_LINK(struct labelLink, lhead);
    DEL_LINK(struct menuLink, head);
    free(path);
    return(0);
}
