/*
  EZinst - Easy Installation -
  Copyright (C) 1993, 1994, 1995 Takashi MANABE
                                     (manabe@Roy.dsl.tutics.tut.ac.jp)

  EZinst is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */

#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include "paths.h"
#include "label.h"
#include "list.h"
#include <jcurses.h>
#include "screen.h"
#include "message.h"

#define	FLOPPY_1ST	1
#define	FLOPPY_2ND	2

#define	MAX_DIRS	20

int yyerror(char *s) {return(1);}
int yywrap() {return(1);}

int categoryLabels, packageLabels;
struct labelLink *labelHead;
char *packageDirs[MAX_DIRS];
short fromFloppy;
char *devFd;
bool japanese;

int SaveFloppyList()
{
    FILE *fp;
    int size, n;
    struct labelLink *llp, *ph;

    for (n = 0, llp = labelHead; llp != NULL; llp = llp->next, n ++) {
	if (n == categoryLabels) ph = llp;
	if (n > categoryLabels && llp->value == TRUE) break;
    }
    if (llp == NULL) return(0);
    unlink(FILE_FLOPPYLIST);
    if ((fp = fopen(FILE_FLOPPYLIST, "w")) == NULL) return(-1);
    fprintf(fp, "# %s\n", devFd);
    for (size = 0; ph != NULL; ph = ph->next) {
	if (ph->value == TRUE) {
	    fprintf(fp, "%s.tgz\n", ph->name);
	    size += ph->size;
	}
    }
    fprintf(fp, "# %d\n", size);
    fclose(fp);
    MessageWin(GetMessage(MSG_MAKE_FLOPPY2), FILE_FLOPPYLIST, 0);
    return(0);
}

int LoadRuleFiles(WINDOW *win)
{
    int i, Eval();
    bool error=FALSE;
    FILE *fp;
    char line[256], buff[2048], *p, *w;
    struct labelLink *llp;

    labelHead = NULL;
    categoryLabels = LoadIntoLabel(FILE_CATEGORIES, &labelHead);
    if (categoryLabels <= 0) {
	wprintw(win, "Error in %s(%d)\n", FILE_CATEGORIES,
		categoryLabels);
	return(-1);
    }
    packageLabels = LoadIntoLabel(FILE_PACKAGES, &labelHead);
    if (packageLabels <= 0) {
	wprintw(win, "Error in %s(%d)\n", FILE_PACKAGES, packageLabels);
	return(-1);
    }

    if ((fp = fopen((japanese ? DESC_JAPANESE:DESC_ENGLISH), "r"))
	== NULL) {
	wprintw(win, "can't open %s(%d)\n",
		japanese ? DESC_JAPANESE:DESC_ENGLISH);
	return(-1);
    }
    buff[0] = '\0';
    llp = NULL;
    while(fgets(line, 100, fp)) {
	if ((p = strpbrk(line, "\r\n")) != NULL) *p = '\0';
	if (line[0] == '#') {
	    if (llp && buff[0] != '\0') {
/*fprintf(stderr, "<%s, %d>\n", llp->name, w - buff);fflush(stderr);*/
		llp->desc = strdup(buff);
	    }
	    llp = GetLabelFromName(&line[1]);
	    w = buff;
	    bzero(buff, 256);
	} else {
	    sprintf(w, "%s\n", line);
	    w += strlen(line) + 1;
	}
    }
    if (llp && buff[0] != '\0') llp->desc = strdup(buff);
    fclose(fp);

    if ((fp = fopen(FILE_SIZE, "r")) == NULL) {
	wprintw(win, "can't open %s\n", FILE_SIZE);
	return(-1);
    }
/*    buff[0] = '\0';*/
    llp = NULL;
    while(fgets(buff, 100, fp)) {
	if (buff[0] == '#') continue;
	sscanf(buff, "%s %d", line, &i);
	for (llp = labelHead; llp != NULL; llp = llp->next)
	    if (llp->size == 0 && !strcmp(llp->name, line)) {
		llp->size = i;
		break;
	    }
	if (llp == NULL) {
	    error = TRUE;
	    wprintw(win, "%s: not defined in rule file\n", line);
	}
    }
    fclose(fp);
    if (error) return(-1);
    if (!fromFloppy) {
	/* install from non-floppy device */
	if ((fp = fopen(FILE_DIRS, "r")) == NULL) {
	    error = TRUE;
	    wprintw(win, FILE_DIRS" %s", GetMessage(MSG_NO_DIRSFILE));
	    wrefresh(win);
	    packageDirs[0] = ".";
	} else {
	    struct stat exist;

	    error = FALSE;
	    i = 0;
	    while(i < MAX_DIRS && fgets(buff, 100, fp)) {
		if (buff[0] == '#') continue;
		if ((p = strpbrk(buff, "\r\n")) != NULL) *p = '\0';
		if (!stat(buff, &exist) && S_ISDIR(exist.st_mode)) {
		    packageDirs[i] = strdup(buff);
		    i ++;
		} else {
		    wprintw(win, "%s ", buff);
		    error = TRUE;
		}
	    }
	    if (error) {
		wprintw(win, ":\n  %s", GetMessage(MSG_DIR_NOTEXIST));
		wrefresh(win);
	    }
	    fclose(fp);
	}
    }
    Eval(labelHead);
    return(error);
}

void main(int argc, char *argv[])
{
    int i;
    char *p, *dir;
    WINDOW *win;
    struct stat exist;

    japanese = TRUE;
    devFd = NULL;
    fromFloppy = 0;
    for (i = 1; i < argc; i ++) {
	p = argv[i];
	if (*p == '-') {
	    p ++;
	    switch(*p) {
	    case 'e':
		japanese = FALSE;
		break;
	    case 'j':
		japanese = TRUE;
		break;
	    case 'f':
		fromFloppy = FLOPPY_1ST;
		if (i + 1 < argc && *argv[i + 1] == '/') {
		    i ++;
		    if (!stat(argv[i], &exist) && S_ISBLK(exist.st_mode))
			devFd = strdup(argv[i]);
		    else perror(argv[i]);
		} else devFd = "/dev/fd0";
		break;
	    }
	} else {
	    if ((p = argv[i] + strlen(argv[i]) - 1) == '/') *p = '\0';
	    dir = argv[i];
	}
    }
    if (dir) chdir(dir);
    if (InitMessage() < 0) exit(1);
    InitScreen();
    win = OpenWin(LINES/2, COLS*3/4, LINES/4 - 1, COLS/8 - 1, 0, 0,
		  0, 0, "Read files", GetMessage(MSG_PRESSKEY));
    scrollok(win, TRUE);
    if ((i = LoadRuleFiles(win)) != 0) {
	wrefresh(win);
	getchar();
    }
    CloseWin(win);
    if (fromFloppy) {
	if (!stat(FILE_FLOPPYLIST, &exist)) {
	    fromFloppy = FLOPPY_2ND;
	    ListLoad(FILE_FLOPPYLIST);
	    MessageWin(GetMessage(MSG_MAKE_FLOPPY3),
			  FILE_FLOPPYLIST, 0);
	} else MessageWin(GetMessage(MSG_MAKE_FLOPPY1),
			  "How to install from floppy disks", 0);
    }
    if (i >= 0) {
	MainMenu();
	if (fromFloppy & FLOPPY_1ST) {
	    SaveFloppyList();
	}
	CloseMessage();
    }
    endwin();
    exit(0);
}
