/*
  EZinst - Easy Installation -
  Copyright (C) 1993, 1994, 1995 Takashi MANABE
                                     (manabe@Roy.dsl.tutics.tut.ac.jp)

  EZinst is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */

#include <stdio.h>
#include "label.h"
#include <jcurses.h>
#include "menu.h"
#include "screen.h"
#include "buffer.h"
#include "message.h"

static
struct menuLink *MenuDisplay(WINDOW *win, struct menuLink *head,
			     int from, int til, int cur, int mode)
{
    int i, y;
    struct menuLink *mlp, *ret;

    ret = NULL;
    for (i=y=0, mlp = head; mlp != NULL && i < til;
	 mlp = mlp->next, i ++)
	if (i >= from) {
#if 0
	    if (mlp->selected == TRUE) wstandout(win);
#endif
	    wmove(win, y ++, 0);
	    if (i == cur) {
		if (!(mode & MENU_SELECT)) wstandout(win);
		ret = mlp;
	    }
	    if (mode & MENU_SELECT)
		wprintw(win, mlp->able ? "[%c] ": " %c  ", mlp->key);
	    wprintw(win, "%-*s", win->maxx - 4, mlp->desc);
	    if (!(mode & MENU_SELECT)) {
		wprintw(win, " (%c)", mlp->key);
		wstandend(win);
	    }
	}
    wmove(win, cur - from, (mode & MENU_SELECT) ? 1: win->maxx - 2);
    wrefresh(win);
    return(ret);
}

int MenuKey(WINDOW *win, struct menuStat *mst, int max, int mode)
{
    bool rew=TRUE;
    int key, i, diff=0;
    struct menuLink *tmp;

    mst->cmp = MenuDisplay(win, mst->head,
			   mst->from, mst->til, mst->cur, mode);
    while(1) {
	key =  (mode & MENU_NEXT) ? KEY_DOWN: GetKey(win);
	for (i = 0, tmp = mst->head; tmp != NULL && tmp->key != key;
	     tmp = tmp->next, i ++);
	if (!(mode & MENU_SELECT) &&
	    tmp && tmp->key == key && tmp->able == TRUE) {
	    mst->cur = i;
	    if (mst->cur < mst->from) {
		mst->til -= mst->from - mst->cur;
		mst->from = mst->cur;
	    }
	    if (mst->cur >= mst->til) {
		mst->from += mst->cur - mst->til + 1;
		mst->til = mst->cur + 1;
	    }
	    mst->cmp = tmp;
	    key = '\r';
	}
/*	printw("<%04X>\n", key);refresh();*/
	diff = 0;
	switch(key) {
	case KEY_PGDOWN:
	    diff = win->maxy;
	    if (mst->cur + diff > max)
		if ((diff = max - mst->cur) == 0) break;
/*	    printw("<%d(%d) + %d:%d>\n", mst->from, win->maxy, diff, max);refresh();*/
	case KEY_CTRL('N'):
	case KEY_DOWN:
	    if (diff == 0) diff = 1;
	    while (mst->cmp->next && mst->cmp->next->able == FALSE) {
		mst->cmp = mst->cmp->next;
		diff ++;
	    }
	    if (!mst->cmp->next) break;
	    if (mst->cur + diff < max) {
		mst->cur += diff;
		if (mst->cur >= mst->til) {
		    if (mst->til + diff < max) {
			mst->til += diff;
			mst->from += diff;
		    } else {
			mst->til = max;
			mst->from = max - win->maxy;
		    }
		}
		rew = TRUE;
		break;
	    }
	    break;
	case KEY_PGUP:
	    diff = win->maxy;
	    if (mst->cur - diff < 0) break;
/*	    printw("<%d - %d>\n", mst->cur, diff);refresh();*/
	case KEY_CTRL('P'):
	case KEY_UP:
	    if (diff == 0) diff = 1;
	    while (mst->cmp->prev && mst->cmp->prev->able == FALSE) {
		mst->cmp = mst->cmp->prev;
		diff ++;
	    }
	    if (!mst->cmp->prev) break;
	    if (mst->cur - diff >= 0) {
		mst->cur -= diff;
		if (mst->cur < mst->from) {
		    mst->til -= diff;
		    mst->from -= diff;
		}
		rew = TRUE;
		break;
	    }
	    break;
	case 'q':
	    if (!(mode & MENU_SELECT)) break;
	case 0x7F:
	case 0x08:
	    return(-1);
	case '\r':
	    if (!(mode & MENU_SELECT) && mst->cmp->p.run
		&& (mst->cmp->p.run(mst->cmp) == 0)) break;
	    else return(0);
	case KEY_CTRL('L'):
	    RefreshAll();
	    break;
	case KEY_CTRL('<'):
	    mst->cur = mst->from = 0;
	    break;
	case '>':
/*	    mst->from = max - mst->til;*/
	    mst->cur = max - 1;
	    mvwprintw(stdscr, 20, 0, "<%d, %d>", mst->from, mst->cur);
	    refresh();
	    break;
	}
	if (rew == TRUE)
	    mst->cmp = MenuDisplay(win, mst->head,
				   mst->from, mst->til, mst->cur, mode);
	if (mode & MENU_SELECT) return(key);
    }
}

struct menuLink *MenuWin(struct menuLink *head,
			 char *title, int top, int left)
{
    int y=0, x=0, x2, max;
    struct menuLink *mlp;
    struct menuStat mst;
    WINDOW *win;

    x = strlen(title);
    for (mlp = head; mlp != NULL; y ++) {
	x2 = strlen(mlp->desc);
	if (x2 > x) x = x2;
	mlp = mlp->next;
    }
    x += 4;	/* ץ */
    max = y;
    if (y > LINES - 2 - top) y = LINES - 2 - top;
    if (x > COLS - 2 - left) x = COLS - 2 - left;
    win = OpenWin(y, x, top, left, 0, 0, 0, 0, title, "");
    mst.head = head;
    mst.cur = mst.from = 0;
    mst.til = y;
    while (MenuKey(win, &mst, max, MENU_NORMAL) == -1);
    CloseWin(win);
    return(mst.cmp);
}

int PackageMenu(struct menuLink *pmp)
{
    struct menuLink *mlp, *tail, *head;
    extern int
	NaviStart(), RuleStart(), ManualStart(),
	ListCatalog(), ResetList();

    head = tail = NULL;
#ifdef	CAN_NAVIGATOR
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_NAVIGATOR);
    mlp->key = 'n';
    mlp->able = TRUE;
    mlp->p.run = NaviStart;
#endif
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_LIST);
    mlp->key = 'l';
    mlp->able = TRUE;
    mlp->p.run = ListCatalog;
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_RULE);
    mlp->key = 'r';
    mlp->able = TRUE;
    mlp->p.run = RuleStart;
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_MANUAL);
    mlp->key = 'm';
    mlp->able = TRUE;
    mlp->p.run = ManualStart;
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_RESET);
    mlp->key = 'i';
    mlp->able = TRUE;
    mlp->p.run = ResetList;
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_QUIT);
    mlp->key = 'q';
    mlp->able = TRUE;
    MenuWin(head, pmp->desc, 1, 2);
    DEL_LINK(struct menuLink, head);
    return(0);
}

#ifdef	INSTALL_SOURCE

int SourceMenu(struct menuLink *pmp)
{
    struct menuLink *mlp, *tail, *head;
    extern int NaviStart(), RuleStart(), ManualStart();

    head = tail = NULL;
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_SRC_FLOPPY);
    mlp->key = 'f';
    mlp->able = TRUE;
    mlp->p.run = NaviStart;
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_SRC_MOUNTED);
    mlp->key = 'p';
    mlp->able = TRUE;
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_SRC_NFS);
    mlp->key = 'n';
    mlp->able = TRUE;
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_SRC_DEV);
    mlp->key = 'c';
    mlp->able = TRUE;
/*    mlp->p.run = ManualStart;*/
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_QUIT);
    mlp->key = 'q';
    mlp->able = TRUE;
    MenuWin(head, pmp->desc, 1, 2);
    DEL_LINK(struct menuLink, head);
    return(0);
}

#endif	INSTALL_SOURCE

int MainMenu()
{
    struct menuLink *mlp, *tail, *head;
    extern int InstallMenu(), AllQuit();

    head = tail = NULL;
#ifdef	INSTALL_SOURCE
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_SOURCE);
    mlp->key = 's';
    mlp->able = TRUE;
    mlp->p.run = SourceMenu;
#endif	INSTALL_SOURCE
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_SELECT);
    mlp->key = 'p';
    mlp->able = TRUE;
    mlp->p.run = PackageMenu;
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_INSTALL);
    mlp->key = 'i';
    mlp->able = TRUE;
    mlp->p.run = InstallMenu;
    NEW_LINK(struct menuLink, head, tail, mlp);
    mlp->desc = GetMessage(MSG_EXIT);
    mlp->key = 'q';
    mlp->able = TRUE;
    mlp->p.run = AllQuit;
    MenuWin(head, GetMessage(MSG_MAIN), 0, 0);
    DEL_LINK(struct menuLink, head);
    return(0);
}
