/*
  EZinst - Easy Installation -
  Copyright (C) 1993, 1994, 1995 Takashi MANABE
                                     (manabe@Roy.dsl.tutics.tut.ac.jp)

  EZinst is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */

#include <stdio.h>
#include <string.h>
#include "paths.h"
#include "list.h"
#include "label.h"
#include "screen.h"
#include "menu.h"
#include "buffer.h"
#include "message.h"
#include <jcurses.h>

extern int categoryLabels, packageLabels;
extern struct labelLink *labelHead;

#define	DEL_MENULINK(head)	\
{\
    struct menuLink *_tmp, *_next;\
\
    _tmp = head;\
    while(_tmp) {\
	_next = _tmp->next;\
	free(_tmp->desc);\
	free(_tmp);\
	_tmp = _next;\
    }\
    head = NULL;\
}

#define	LEN_PNAME	20

int AllQuit(struct menuLink *pmp)
{
    struct labelLink *llp;

    for (llp = labelHead; llp != NULL; llp = llp->next)
	if (llp->value == TRUE) {
	    if (MessageWin(GetMessage(MSG_SAVE_LIST), "", 0) == 'y')
		ListSave(labelHead, SAVE_INSTALL);
	    break;
	}
    return(-1);
}

static int CountSize(struct labelLink *head)
{
    int size=0;
    struct labelLink *llp;

    for (llp = head; llp != NULL; llp = llp->next)
	if (llp->value == TRUE) size += llp->size;
    return(size);
}

int SelectWin(char *msg, struct menuLink *mhead, char *title,
	      int x, int mode)
{
    int y=0, x2, y0, max, ret, ym, sx, sy, n, ms, mr, ss;
    char *p, *w;
    struct menuStat mst;
    struct menuLink *mlp;
    WINDOW *win;
    extern bool japanese;

    if ((x2 = strlen(title)) > x) x = x2;
    w = msg;
    do {
	if ((p = strchr(w, '\n')) != NULL) *p = '\0';
	x2 = strlen(w);
	if (x2 > x) x = x2;
	if (p) {
	    y ++;
	    *p = '\n';
	    w = p + 1;
	}
    } while(p);
    if (y > 0) y ++;
    y0 = y;
    for (mlp = mhead; mlp != NULL; y ++) {
	x2 = strlen(mlp->desc);
	if (x2 > x) x = x2;
	mlp = mlp->next;
    }
    max = y - y0;
    if (y > LINES - 2) y = LINES - 2;
    sx = (COLS - x - 4)/2;
    sy = (LINES - y - 2)/2;
    win = OpenWin(y, x, sy, sx, 0, x - LEN_PNAME + 2, y0, 0,
		  title, GetMessage(MSG_BACKSPACE));
    MessageDisp(win->orig, 0, y0, win->maxx, 1, 2, msg);
    if (japanese) {
	if (y0) {
	    mvwprintw(win->orig, y0, 0, "");
	    for (n = 2; n < win->orig->maxx - 2; n +=2)
		wprintw(win->orig, "");
	    wprintw(win->orig, "");
	}
	mvwprintw(win->orig, y0, LEN_PNAME, "");
	for (n = y0 + 1; n <= y; n ++)
	    mvwprintw(win->orig, n, LEN_PNAME, "");
	mvwprintw(win->orig, y + 1, LEN_PNAME, "");
    } else {
	wstandout(win->orig);
	if (y0) mvwprintw(win->orig, y0, 0, "%*s", win->orig->maxx, "");
	for (n = y0 + 1; n <= y; n ++)
	    mvwprintw(win->orig, n, LEN_PNAME, " ");
	wstandend(win->orig);
    }
    mst.cmp = mst.head = mhead;
    mst.cur = mst.from = 0;
    mst.til = y - y0;
    ret = '\0';
    mr = ym = y - y0;
    x -= win->maxx + 2;
    ms = 0;
    ss = win->orig->maxx - 16;
    if (mode != SELECT_LIST)
	mvwprintw(win->orig, y0, ss, "%7d [KB]",
		  CountSize(labelHead) / 1024);
    p = "";
    do {
	switch(ret) {
	case ' ':
	    mst.cmp->p.lbl->selected =
		(mst.cmp->p.lbl->value) ? FALSE: TRUE;
	    mst.cmp->p.lbl->value = mst.cmp->p.lbl->selected;
	    if (mode == SELECT_RULE) Eval(labelHead);
	    if (mode != SELECT_LIST)
		mvwprintw(win->orig, y0, ss, "%7d [KB]",
			  CountSize(labelHead) / 1024);
	    for (mlp = mhead; mlp != NULL; mlp = mlp->next) {
		mlp->able =
		    (mode == SELECT_RULE) ? mlp->p.lbl->able: TRUE;
		mlp->key = (mlp->p.lbl->value == TRUE) ? 'O': ' ';
	    }
	    MenuKey(win, &mst, max, MENU_NEXT|MENU_SELECT);
	    break;
	case KEY_RIGHT:
	    if (*p) ms ++;
	    break;
	case KEY_LEFT:
	    if (ms > 0) ms --;
	    break;
	default:
	    ms = 0;
	}
	p = MessageDisp(win->orig, ms, ym, x, y0 + 1, LEN_PNAME + 2,
		    mst.cmp->p.lbl->desc);
    } while ((ret = MenuKey(win, &mst, max, MENU_SELECT)) > 0);
    CloseWin(win);
    return(ret);
}

int RuleStart(struct menuLink *pmp)
{
    int i, ret;
    char desc[LEN_PNAME];
    struct labelLink *llp;
    struct menuLink *mlp, *tail, *head;

    head = tail = NULL;
    for (i = 0, llp = labelHead; llp != NULL; llp = llp->next, i ++) {
	if (*(llp->name) == '$') continue;
	NEW_LINK(struct menuLink, head, tail, mlp);
	if (i < categoryLabels)
	    sprintf(desc, "%-8s", llp->name);
	else
	    sprintf(desc, "%-8s %5d", llp->name, llp->size / 1024);
	mlp->desc = strdup(desc);
	mlp->p.lbl = llp;
	mlp->able = llp->able;
	mlp->key = (llp->value == TRUE) ? 'O': ' ';
    }
    ret = SelectWin("", head, GetMessage(MSG_PKG_SIZE), 66, SELECT_RULE);
    DEL_MENULINK(head);
    return(0);
}

int ManualStart(struct menuLink *pmp)
{
    int i, ret;
    char desc[LEN_PNAME];
    struct labelLink *llp;
    struct menuLink *mlp, *tail, *head;

    head = tail = NULL;
    for (i = 0, llp = labelHead; llp != NULL; llp = llp->next, i ++) {
	if (i < categoryLabels) continue;
	if (*(llp->name) == '$') continue;
	NEW_LINK(struct menuLink, head, tail, mlp);
	sprintf(desc, "%-8s %5d", llp->name, llp->size / 1024);
	mlp->desc = strdup(desc);
	mlp->able = TRUE;
	mlp->p.lbl = llp;
	mlp->key = (llp->value == TRUE) ? 'O': ' ';
    }
    ret = SelectWin("", head, "", 66, SELECT_FREE);
    DEL_MENULINK(head);
    return(0);
}    

int ResetList(struct menuLink *pmp)
{
    struct labelLink *llp;

    for (llp = labelHead; llp != NULL; llp = llp->next)
	llp->value = llp->selected = FALSE;
    return(0);
}
