/*
  EZinst - Easy Installation -
  Copyright (C) 1993, 1994, 1995 Takashi MANABE
                                     (manabe@Roy.dsl.tutics.tut.ac.jp)

  EZinst is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */

#include <unistd.h>
#include <malloc.h>
#include <string.h>
#include <jcurses.h>
#include "version.h"
#include "screen.h"
#include "message.h"
#include "paths.h"

struct winLink {
    WINDOW *win;
    struct winLink *next;
};

static struct winLink *winHead, *winTail;
extern bool japanese;

static void AddWinLink(WINDOW *win)
{
    struct winLink *new;

    new = calloc(sizeof(struct winLink), 1);
    if (winTail == NULL)
	winHead = new;
    else
	winTail->next = new;
    winTail = new;
    new->win = win;
}

static void DelWinLink(WINDOW *win)
{
    struct winLink *cur, *prev;

    for (cur = winHead; cur != NULL; cur = cur->next) {
	if (cur->win == win) {
	    prev->next = cur->next;
	    if (winTail == cur) winTail = prev;
	    free(cur);
	    return;
	}
	prev = cur;
    }
}

void RefreshAll()
{
    struct winLink *tmp;

/*    clear();*/
    for (tmp = winHead; tmp != NULL; tmp = tmp->next) {
	touchwin(tmp->win);
	wrefresh(tmp->win);
    }
}

/*


*/

static void MakeFrame(WINDOW *win, char *title, char *tail, int dy, int dx)
{
    int i;

    if (japanese) {
	mvwprintw(win, 0, 0, "");
	for (i = 2; i < dx - 2; i += 2) wprintw(win, "");
	wprintw(win, "");
	for (i = 1; i < dy - 1; i ++) {
	    mvwprintw(win, i, 0, "");
	    mvwprintw(win, i, dx - 2, "");
	}
	mvwprintw(win, dy - 1, 0, "");
	for (i = 2; i < dx - 2; i += 2) wprintw(win, "");
	wprintw(win, "");
	if ((i = strlen(title)) > 0)
	    mvwprintw(win, 0, 2, "<%-*s>", ((i+1)/2)*2, title);
	if ((i = strlen(tail)) > 0)
	    mvwprintw(win, dy - 1, dx - ((i+1)/2)*2 - 4, "(%-*s)",
		      ((i+1)/2)*2, tail);
	else wmove(win, 0, 0);
    } else {
	wstandout(win);
	wprintw(win, "%*s", dx, "");
	for (i = 1; i < dy - 1; i ++) {
	    mvwprintw(win, i, 0, " ");
	    mvwprintw(win, i, dx - 1, " ");
	}
	mvwprintw(win, dy - 1, 0, "%*s", dx, "");
	for (i = 0; i < dy; i ++) wprintw(win, "%*s", dx, "");
	mvwprintw(win, 0, 2, title);
	mvwprintw(win, dy - 1, 0, "%*s", dx - 2, tail);
	wstandend(win);
    }
    wrefresh(win);
}    

WINDOW *OpenWin(int dy, int dx, int sy, int sx, int left, int right,
		int top, int bottom, char *head, char *tail)
{
    int y;
    WINDOW *win, *pwin;

    pwin = newwin(dy + 2, dx + 4, sy, sx);
    AddWinLink(pwin);
    MakeFrame(pwin, head, tail, dy + 2, dx + 4);
    win = subwin(pwin, dy - top - bottom, dx - left - right,
		 sy + top + 1, sx + left + 2);
    AddWinLink(win);
    for (y = 0; y < dy - 2; y ++) wprintw(win, "%*s", dx - 2, " ");
    wmove(win, 0, 0);
    return(win);
}

void CloseWin(WINDOW *win)
{
    WINDOW *pwin;

    pwin = win->orig;
    delwin(win);
    DelWinLink(win);
    wclear(pwin);
    wrefresh(pwin);
    delwin(pwin);
    DelWinLink(pwin);
    RefreshAll();
}

char *MessageDisp(WINDOW *win, int sy, int dy, int dx, int y, int x,
		 char *msg)
{
    char *p;
    int i, n;

    i = -sy;
    for (n = 0; i < dy; i ++) {
	if (msg && *msg) {
	    if ((p = strchr(msg, '\n')) != NULL) *p = '\0';
	    if (i >= 0) mvwprintw(win, y + i, x, "%-*s", dx, msg);
	    n ++;
	    if (p) {
		*p = '\n';
		msg = p + 1;
	    }
	} else if (i >= 0) mvwprintw(win, y + i, x, "%-*s", dx, "");
    }
    wrefresh(win);
    return(msg);
}

int MessageWin(char *msg, char *head, int timeout)
{
    int y=0, x=0, x2;
    char *p, *w;
    WINDOW *win;

    x = strlen(head) + 2;
    p = GetMessage(MSG_PRESSKEY);
    if (strlen(p) + 2 > x) x = strlen(p) + 2;
    w = msg;
    do {
	if ((p = strchr(w, '\n')) != NULL) *p = '\0';
	x2 = strlen(w);
	if (x2 > x) x = x2;
	if (p) {
	    y ++;
	    *p = '\n';
	    w = p + 1;
	}
    } while(p);
    x = ((x + 1) / 2) * 2;
    win = OpenWin(y, x, (LINES - y)/2 - 1, (COLS - x)/2 - 1, 0, 0,
		  0, 0, head, timeout ? "": GetMessage(MSG_PRESSKEY));
    MessageDisp(win, 0, y, x, 0, 0, msg);
    SET_CURSOR(win);
    if (timeout) sleep(timeout);
    else x2 = getch();
    CloseWin(win);
    return(x2);
}

unsigned GetKey(WINDOW *win)
{
    unsigned key, key0;
    enum {ESC0, ESC1, ESC5B, ESCPG} esc=ESC0;

#if 0
    move(0, 0);
    printw("                  \r");
#endif
    while(1) {
	key = wgetch(win);
#if 0
	printw("<%04X>", key);
	refresh();
#endif
	switch(esc) {
	case ESC0:
	    if (key == 0x1B) esc = ESC1;
	    else return(key);
	    break;
	case ESC1:
	    if (key == 0x5B) esc = ESC5B;
	    else return(key);
	    break;
	case ESC5B:
	    switch(key) {
	    case 0x41:
	    case 0x42:
	    case 0x43:
	    case 0x44:
		return(KEY_CURSOR|key);
	    case 0x35:
	    case 0x36:
		key0 = key | KEY_CURSOR;
		esc = ESCPG;
		break;
	    }
	    break;
	case ESCPG:
	    return(key0);
	    break;
	}
    }
}

void InitScreen()
{
    FILE *fp;
    int build=999;
    char title[80];

    if ((fp = fopen(FILE_BUILD, "r")) != NULL) {
	fscanf(fp, "%d", &build);
	fclose(fp);
    }
    sprintf(title, "EZinst version "VERSION" rule %03d", build);
    initscr();
    noecho();
    crmode();
    nonl();
    AddWinLink(stdscr);
    MessageWin(GetMessage(MSG_NOWARRENTY), title, 0);
}
